package edu.uoc.tfc.web;

import java.io.ByteArrayInputStream;
import java.io.DataInputStream;
import java.io.InputStream;
import java.io.OutputStream;

import javax.activation.DataSource;

public final class EmktDataSource implements DataSource {

	private final String contentType;
	private final byte[] buf;
	private final int start;
	private final int len;
	private final InputStream is;

	public EmktDataSource(byte[] buf, String contentType) {
		this(buf, 0, buf.length, contentType);
	}

	public EmktDataSource(byte[] buf, int length, String contentType) {
		this(buf, 0, length, contentType);
	}

	public EmktDataSource(byte[] buf, int start, int length,
			String contentType) {
		this.buf = buf;
		this.start = start;
		this.len = length;
		this.contentType = contentType;
		is = null;
	}

	public EmktDataSource(InputStream _is) {
		this.is = _is;
		
		// Not needed
		this.buf = null;
		this.start = 0;
		this.len = 0;
		this.contentType = null;
	}

	public String getContentType() {
		if (contentType == null)
			return "application/octet-stream";
		return contentType;
	}

	/**
	 * El mtodo getInputStream() debe devolver siempre un nuevo InputStream, 
	 * no debe pasarsele un InputStream por pase de parmetros
	 * 
	 * @see http://bugs.sun.com/bugdatabase/view_bug.do?bug_id=4267294
	 */
	public InputStream getInputStream() {
		if (is != null) {
			return new DataInputStream(is);
		} else {
			return new ByteArrayInputStream(buf, start, len);
		}
	}

	public String getName() {
		return null;
	}

	public OutputStream getOutputStream() {
		throw new UnsupportedOperationException();
	}

}
