package edu.uoc.tfc.web.tags;

import java.lang.reflect.Method;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.PageContext;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;
import org.apache.struts.config.ActionConfig;

import com.sdicons.json.mapper.JSONMapper;
import com.sdicons.json.model.JSONValue;

import edu.uoc.tfc.gui.common.JSONDataTableInitializer;
import edu.uoc.tfc.gui.common.JSONDataTableResult;

public class DataTablesAjaxResponseTag extends EmktBaseTag {

    private static final long serialVersionUID = 1L;

	private static final Logger LOG = Logger.getLogger(DataTablesAjaxResponseTag.class);
	
    private PageContext pageContext;
    private String columnNames;
    private String columnIdName;

    public String getColumnNames() {
		return columnNames;
	}

	public void setColumnNames(String columnNames) {
		this.columnNames = columnNames;
	}

	public String getColumnIdName() {
		return columnIdName;
	}

	public void setColumnIdName(String columnIdName) {
		this.columnIdName = columnIdName;
	}

	/**
     * Returns pageContext.
     *
     * @return pageContext
     */
    public PageContext getPageContext() {
        return this.pageContext;
    }

    /**
     * Setear el contexto de la pagina.
     *
     * @param pageContext el contexto de la pagina.
     * @see javax.servlet.jsp.tagext.Tag#setPageContext(javax.servlet.jsp.PageContext)
     */
    public void setPageContext(PageContext pageContext) {
        this.pageContext = pageContext;
    }


	@Override
	public int doStartTag() throws JspException {
        try {
        	ActionConfig actionInstance =
                    (ActionConfig) this.getPageContext().
                    	getRequest().getAttribute("org.apache.struts.action.mapping.instance");
    		
    		
        	ActionForm formBean = 
        			(ActionForm) this.getPageContext().
        				getRequest().getAttribute(actionInstance.getName());
        	
    		Object objeto = this.parseaObjeto(formBean);
    		
    		if (StringUtils.isNotEmpty(this.getColumnIdName())) {
    			JSONDataTableInitializer.parseAndInitialize(objeto, this.getColumnIdName());
    		}
    		
    		JSONObject jsonObject = new JSONObject();
    		jsonObject.setAaData(objeto);
    		jsonObject.setiTotalDisplayRecords("0");
    		jsonObject.setiTotalRecords("0");
    		
			// Recuperamos de la request el valor sEcho que nos proporciona el
			// framework datatables. Esto lo utiliza datatables para identificar
			// las peticiones de datos, de forma que pueda distinguir las ms
			// recientes de las ms antiguas y decidir a partir de este valor
			// los datos a mostrar.
    		String sEcho = getPageContext().getRequest().getParameter("sEcho");
    		if (StringUtils.isNotBlank(sEcho))
    		{
    			jsonObject.setsEcho(new Integer(sEcho));
    		}

    		if (objeto instanceof JSONDataTableResult) {
    			JSONDataTableResult datatableResult = (JSONDataTableResult)objeto;
    			if (datatableResult.getElements() != null) {
    				jsonObject.setAaData(datatableResult.getElements().toArray());
    			}
    			jsonObject.setiTotalRecords(String.valueOf(datatableResult.getTotalRecords()));
    			jsonObject.setiTotalDisplayRecords(String.valueOf(datatableResult.getTotalRecords()));
    		}
    		else if (objeto instanceof Object[]) {
    			Object[] objectArray = (Object[])objeto;
    			int length = objectArray.length;
    			jsonObject.setiTotalRecords(String.valueOf(length));
    			jsonObject.setiTotalDisplayRecords(String.valueOf(length));
    		}
    		
    		JSONValue jsonValue = JSONMapper.toJSON(jsonObject);
            
            JspWriter out = this.getPageContext().getOut();
            this.getPageContext().getResponse().setContentType("application/json");
            out.write(jsonValue.render(true));
			
		} catch (Exception e) {
			LOG.error("Error parseando objeto a JSON: " + this.getProperty(), e);
			throw new JspException("Error parseando objeto a JSON: " + this.getProperty());
		} 

		
		return SKIP_BODY;
	}
	
    private Object parseaObjeto(ActionForm formBean)throws Exception {
    	
    	Method[] metodos = formBean.getClass().getMethods();
    	Method getter = null;
    	
    	for (int i = 0; i < metodos.length; i++) {
			if (metodos[i].getName().equalsIgnoreCase("get" + this.getProperty())) {
				getter = metodos[i];
				break;
			}
		}
    	
    	Object returnValue = getter.invoke(formBean, new Object[]{});
		return returnValue;
	}

	public int doEndTag() {
        return EVAL_PAGE;
    }

	public static class JSONObject {
		
		private Integer sEcho;
		public Integer getsEcho() {
			return sEcho;
		}
		public void setsEcho(Integer sEcho) {
			this.sEcho = sEcho;
		}
		public Object getAaData() {
			return aaData;
		}
		public void setAaData(Object aaData) {
			this.aaData = aaData;
		}
		public String getiTotalRecords() {
			return iTotalRecords;
		}
		public void setiTotalRecords(String iTotalRecords) {
			this.iTotalRecords = iTotalRecords;
		}
		public String getiTotalDisplayRecords() {
			return iTotalDisplayRecords;
		}
		public void setiTotalDisplayRecords(String iTotalDisplayRecords) {
			this.iTotalDisplayRecords = iTotalDisplayRecords;
		}
		private Object aaData;
		private String iTotalRecords;
		private String iTotalDisplayRecords;
	}
	
}
