package edu.uoc.tfc.web.tags;



import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspWriter;
import javax.servlet.jsp.tagext.Tag;

import org.apache.commons.lang.StringUtils;
import org.apache.log4j.Logger;



public class DataTablesButtonTag extends AbstractButtonTag
{



    private static final long serialVersionUID = 1L;

    

	private static final Logger LOG = Logger.getLogger(DataTablesButtonTag.class);



	private String actionType;
	private boolean showOnModal = false;
	private String columnName;

	

	public String getColumnName() {
		return columnName;
	}

	public void setColumnName(String columnName) {
		this.columnName = columnName;
	}



	@Override
	public String getTargetDiv() {
		if (StringUtils.isBlank(targetDiv)) {
			return "content";
		} else return targetDiv;
	}



    public String getActionType() {
		return actionType;
	}

	public void setActionType(String actionType) {
		this.actionType = actionType;
	}



	public boolean isShowOnModal() {
		return showOnModal;
	}

	public void setShowOnModal(boolean showOnModal) {
		this.showOnModal = showOnModal;
	}



	public int doStartTag() throws JspException
    {
          if (
             (isModal() && isShowOnModal())
             ||
             (!isModal() && !isShowOnModal())
          ){
                 try
                 {
                        Tag parentTag = this;
                        boolean found = false;
                        int counter = 0;
                        do
                        {
                               parentTag = parentTag.getParent();
                               found = parentTag instanceof DataTablesTag;
                        }
                        while ((!found && counter++ < EmktBaseTag.MAX_DEEP && parentTag != null));

                        if (parentTag instanceof DataTablesTag)
                        {
                               writeButton(pageContext.getOut());
                        }
                        else
                        {
                               if (LOG.isDebugEnabled())
                               {
                                     LOG.debug("No est dentro de un Datatable");
                               }
                        }
                 }
                 catch (Exception ex)
                 {
                        String message = "Error finalizando Botn de DataTable: " + ex.getMessage();
                        LOG.error(message, ex);
                        throw new JspException(message);
                 }
          }

          return SKIP_BODY;
    }



	@Override
	protected void writeButton(JspWriter out) throws Exception
	{
		String theId = "";
		if (StringUtils.isNotBlank(getId()))
		{
			theId = "id=\"" + getId() + "\" ";
		}
		write(out, "<button " + theId + getTypeMode() + getClassMode() + getData() + ">");
		write(out, "<div class=\"the-icons\">");
		write(out, "<i class=\"" + getIconClass() + "\"></i> ");
		write(out, getMessage(getLabel(), null));
		write(out, "</div>");
		write(out, "</button>");
	}



	@Override
	protected String getTypeMode()
	{
		StringBuilder typeMode = new StringBuilder(" ");
		if (!StringUtils.isBlank(getModalClass()))
		{
			typeMode.append("data-toggle=\"" + getModalClass() + "\" ");
		}
		else
		{
			typeMode.append("type=\"submit\"");
		}
		return typeMode.toString();
	}



	@Override
	protected String getClassMode()
	{
		return " class=\"btn " + getStyle() + " emkt-datatable-button\"";
	}



	@Override
	protected String getData()
	{
//		data-service-url="http://localhost:8080/tfc-project" 
//		data-operation-url="/CUGestioUsuaris/CUGestioUsuaris.do"
		
		StringBuilder data = new StringBuilder();
		if (StringUtils.isNotBlank(getOperation()))
		{
			
			data.append(
				" data-service-url=\"" + super.getContextPath() + "\"" +
				" data-operation-url=\"" + getOperation() + "\""
			);
		}

		if (StringUtils.isNotBlank(getModalClass()))
		{
			data.append(" data-toggle=\"" + getModalClass() + "\"");
			data.append(" data-target-div=\"modal-body\"");
		}
		else
		{
			data.append(" data-target-div=\"" + getTargetDiv() + "\"");
		}

		data.append(" data-readonly=\"" + isReadonly() + "\"");
		data.append(" data-bytoken=\"" + getBytoken() + "\"");

		String actionScript = getDataActionScript();
		if (StringUtils.isNotBlank(actionScript))
		{
			data.append(" data-action=\"" + actionScript + "\"");
		}

		if (StringUtils.isNotBlank(getColumnName()))
		{
			data.append(" data-columnName=\"" + getColumnName() + "\"");
		}

		return data.toString();
	}



	private String getDataActionScript()
	{
		String result = null;
		if (StringUtils.isBlank(actionType))
		{
			String selectFunction = pageContext.getRequest().getParameter("select-function");
			if (StringUtils.isNotBlank(selectFunction))
			{
				result = selectFunction;
			}
		}
		else
		{
			result = getActionType();
		}
		return result;
	}



}
