package edu.uoc.tfc.web.tags;

import javax.servlet.jsp.JspException;
import javax.servlet.jsp.JspWriter;



public class SearchFieldTag extends TextFieldTag
{

	private static final long serialVersionUID = 1L;


    private String service;
    private String operation;
    private boolean searchDisabled = false;



	public String getService() {
    	return service;
    }

	public void setService(String service) {
    	this.service = service;
    }



	public String getOperation() {
    	return operation;
    }

	public void setOperation(String operation) {
    	this.operation = operation;
    }



	public boolean isSearchDisabled() {
	    return searchDisabled;
    }

	public void setSearchDisabled(boolean searchDisabled) {
	    this.searchDisabled = searchDisabled;
    }



	@Override
    protected void doStartFormField(JspWriter out) throws JspException
	{
		try
		{
			if (isModal() || isReadonly() || getForm().isReadonly())
			{
				super.doStartFormField(out);
			}
			else
			{
				StringBuilder classes = new StringBuilder("input-large search-query");
				if (isRequired())
				{
					classes.append(" emkt-required");
				}

				write(out, "<input class=\"" + classes + "\"");
				write(out, " type=\"text\" name=\"" + getName() + "\" id=\"" + getId() + "\" value=\"" + getValue() + "\"");

				if (!isSearchDisabled())
				{
					write(out, " readonly=\"readonly\" />");

					// Boton para limpiar el resultado de la bsqueda
					write(out, "<span class=\"icon " + getCleanButtonClass() + "\" >");
					write(out, "<i class=\"icon-remove\"></i>");
					write(out, "</span>");

					String functionName = "select" + escapeNameForJsFunction(getName());

					write(out,
						"<button type=\"submit\" class=\"btn emkt-ajax\""
						+ " data-service-url=\"" + super.getContextPath() + "\""
						+ " data-operation-url=\"" + getOperation() + "\""
						+ " data-toggle=\"modal-emkt-big\""
						+ " data-select-function=\"" + functionName + "\""
						+ " data-target-div=\"modal-body\">"
					);

					write(out, "<div class=\"the-icons\">");
					write(out, "<i class=\"icon-search\"></i>");
					write(out, "</div>");
					write(out, "</button>");

					doSelectFunctionJavascript(out, functionName);
				}
				else
				{
					write(out, " />");

					write(out, "<span class=\"btn disabled\">");
					write(out, "<div class=\"the-icons\">");
					write(out, "<i class=\"icon-search\"></i>");
					write(out, "</div>");
					write(out, "</span>");
				}
			}
		}
		catch (Exception ex)
		{
			String message = "Error generando Campo de bsqueda: " + ex.getMessage();
			throw new JspException(message);
		}
	}



	protected String getCleanButtonClass() {
		return "emkt-search-clean";
    }



	protected void doSelectFunctionJavascript(JspWriter out, String functionName)
	throws Exception
	{
		write(out, "<script type=\"text/javascript\">");
		write(out, "  function " + functionName + "(row) {");
		write(out, "    $(\"#" + getId() + "\").val(row.id).change();");
		write(out, "    if (typeof " + functionName + "Extended === \"function\"){");
		write(out, "      " + functionName + "Extended();");
		write(out, "    }");
		write(out, "    modal.close();");
		write(out, "    return null;");
		write(out, "  }");
		write(out, "</script>");
	}


	@Override
    protected void doEndFormField(JspWriter out) throws JspException
    {
		// No hacemos nada
    }



}
